clear all
close all
clc

% %  % SET UP 
mData    = xlsread('DataUK.xls');
vPi      = mData(:,3);
vU       = mData(:,11);
mZ       = mData(:,6);
t        = (1:1:size(mData,1))';
t2       = t.^2;
mX       = [t t2 lagmatrix(vPi,1) lagmatrix(vPi,2) lagmatrix(vPi,3) lagmatrix(vPi,4) lagmatrix(vU,1) lagmatrix(vU,2) lagmatrix(vU,3) lagmatrix(vU,4)];
iH       = 25;

% Storage results  
vBeta    = zeros(iH,1);             % storage 2SLS point 
vBetaPi  = zeros(iH,1);             % storage irf pi   
vBetaUr  = zeros(iH,1);             % storage irf ur        
vLowerAR = zeros(iH,1);             % lower AR-bound  
vUpperAR = zeros(iH,1);             % upper AR-bound 
vLowerAR2s = zeros(iH,1);             % lower AR-bound  
vUpperAR2s = zeros(iH,1);             % upper AR-bound 
vLowerPI = zeros(iH,1);             % lower bound AR pi 
vUpperPI = zeros(iH,1);             % upper bound AR pi 
vLowerUR = zeros(iH,1);             % lower bound AR ur
vUpperUR = zeros(iH,1);             % upper bound AR ur 
vFstat   = zeros(iH,1);             % F-stats 
vParB    = (-10:0.01:5)';           % parameters to test 
mWald    = zeros(size(vParB,1),iH); % store Wald
mTest    = zeros(size(vParB,1),iH); % store rejection 

% compute multiplier for h=0,1,2,...
for h=1:iH
    vPiC = vPi;   
    vUC  = vU ;
    if h>1                                       % construct cumulative pi and u 
        for j=2:h
            vPiC = vPiC + lagmatrix(vPi,-(j-1));
            vUC  = vUC  + lagmatrix(vU,-(j-1));
        end
    end 
    
    mObs     = [vPiC vUC mZ mX];                % drop the missing observations 
    mObs     = rmmissing(mObs);
    
    iTc      = size(mObs,1);                    % observations 
    
    mXC      = [ones(iTc,1) mObs(:,4:end)];     % project out constant and control variables    
    mMx      = eye(iTc) - mXC * inv(mXC'*mXC) * mXC'; 
    
    vPiC     = mMx * mObs(:,1);
    vUC      = mMx * mObs(:,2);
    mZC      = [mObs(:,3) mObs(:,3).^2];
    iL       = size(mZC , 2);                  % number of instruments   
    
    % IV reg 
    vCoefs   = inv(vUC' * mZC * inv(mZC'*mZC) * mZC' * vUC) * vUC' * mZC * inv(mZC'*mZC) * mZC' * vPiC; 
    vBeta(h) = vCoefs(1);                      % 2SLS coefs 
    
    
    % standard bounds 
    vRes   = vPiC -  vBeta(h) * vUC;
    mV     = NeweyWest(vRes, mZC ,-1,0);
    mPz    = mZC * inv(mZC' * mZC) * mZC';
    mAvar  = inv(vUC' * mPz * vUC) * vUC' * mZC * inv(mZC' * mZC) * mV * inv(mZC' * mZC) * mZC' * vUC * inv(vUC' * mPz * vUC);
    vLowerAR2s(h) = vBeta(h) - 1.96 * sqrt(mAvar);
    vUpperAR2s(h) = vBeta(h) + 1.96 * sqrt(mAvar);

    for k = 1:size(vParB,1) 
        vRes   = vPiC - vParB(k) * vUC;                       % residual 
        vGamma = mZC' * vRes;       
        vRes1  = vRes - mZC *  inv(mZC'*mZC) *  vGamma;       
        mV     = NeweyWest(vRes1, mZC ,-1,0);                  % dimension 2L x 2L 
        
        mWald(k,h) = vGamma' * inv(mV) * vGamma;               % AR statistic  
      
        if (mWald(k,h) < chi2inv(0.95,iL))
            mTest(k,h) = 1; 
        end
        
    end

     % find upper and lower confidence bounds AR statistic  
    vBounds = FindUpperLower(mTest(:,h), vParB);
    vLowerAR(h) = vBounds(1);
    vUpperAR(h) = vBounds(2);    
   
    % compute effective F-stat 
    vFstat(h)  = EffectiveFstat(vUC, mZC);     
    
    mZC = mZC(:,1);
    
    % Compute IRF PI 
    vBetaPi(h) = inv(mZC'*mZC) * mZC' * vPiC;
    vRes       = vPiC - mZC *  vBetaPi(h);       
    mV         = NeweyWest(vRes, mZC ,4,0); 
    mAvar      =  inv(mZC'*mZC) * mV * inv(mZC'*mZC);
    vLowerPI(h)=  vBetaPi(h) - 1.96 * sqrt(mAvar);
    vUpperPI(h)=  vBetaPi(h) + 1.96 * sqrt(mAvar);
    
    % Compute IRF UR
    vBetaUr(h) = inv(mZC'*mZC) * mZC' * vUC;
    vRes       = vUC - mZC *  vBetaUr(h);       
    mV         = NeweyWest(vRes, mZC ,4,0); 
    mAvar      =  inv(mZC'*mZC) * mV * inv(mZC'*mZC);
    vLowerUR(h)=  vBetaUr(h) - 1.96 * sqrt(mAvar);
    vUpperUR(h)=  vBetaUr(h) + 1.96 * sqrt(mAvar);
   
end
  

% MAKE FIGURE 
kappah=  vBeta;
kappahinit = [vBeta(1:6) ; NaN(iH-6,1)];
kappahub=vUpperAR;
kappahlb=vLowerAR;

error1 = abs(vBeta - kappahlb);
error2 = abs(vBeta - kappahub);

x=(1:length(kappah))';

figure, 
sbp=subplot(223), hold on
bar(x,vFstat)
ylabel('F-stat','Interpreter','latex')
xlim([1 iH]);
ylim([0 12]);
set(gca,'Xtick',1:5:iH+1), xlim([1 iH+1]),
ax=get(sbp,'Position');
ax(2)=ax(2)-.0;
ax(4)=ax(4)-0.20;
set(sbp,'Position',ax);
set(gca,'Xtick',1:5:iH+1), xlim([1 iH]),
set(gca','Xticklabel',num2str((0:5:iH+1)'))
xlabel('Quarters','Interpreter','latex')

sbp2=subplot(221), hold on
plot(x,0*kappah ,'LineWidth',1,'Color',[.7 .7 .7],'LineStyle','--');
H1= shadedErrorBar_RB(x,vBeta,[error2 error1], 'k',0,1);
g(1)=plot( kappah,'Linewidth',2,'Color','k','LineStyle','-');
g(1)=plot(vLowerAR2s,'Linewidth',1,'Color','k','LineStyle',':');
g(1)=plot(vUpperAR2s,'Linewidth',1,'Color','k','LineStyle',':');
ylabel('$\mathcal{ P}_h$','Interpreter','latex','Fontsize',14)
ylim([-6 5])
xlim([1 iH]);
set(gca','Xticklabel',num2str((0:5:iH+1)'))
set(gca,'Xtick',1:5:iH+1), xlim([1 iH]),
set(gca,'Ytick',-14:2:14), xlim([1 iH]),
ax=get(sbp2,'Position');
ax(2)=ax(2)-.217;
ax(4)=ax(4)+0.22; %or wathever
set(sbp2,'Position',ax);
title('Phillips multiplier','Interpreter','latex')


%rescale IRFs so that ir of ur peaks at 1:
ir2sls = [vBetaUr vBetaPi]  / max(vBetaUr);
ir     = [vBetaUr vBetaPi] / max( vBetaUr); 
irub   = [ vUpperUR vUpperPI ]/ max(vBetaUr);
irlb   = [ vLowerUR vLowerPI ]/ max(vBetaUr);

error1 = abs(ir - irlb);
error2 = abs(ir - irub);

iH=length(ir);
x=(1:iH)';

figure(1),
kk=0;

for ii=2:2:size(ir,2)*2
    subplot(2,2,ii), hold on
    kk=kk+1;
    plot(x,0*ir(:,kk) ,'LineWidth',1,'Color',[.7 .7 .7],'LineStyle','--');
    H1= shadedErrorBar(x,ir(:,kk),[error2(:,kk) error1(:,kk)], 'k');
    g(1)=plot(ir(:,kk),'Linewidth',2,'Color','k','LineStyle','-');
    if kk==1
        ylabel('{$\mathcal R^{\bar u}_h$}','Interpreter','latex','Fontsize',14);
    title('Impulse Responses','Interpreter','latex')  
    end
    if kk==2
        ylabel('{$\mathcal R^{\bar \pi}_h$}','Interpreter','latex','Fontsize',14);
    end
    if kk==2
        xlabel('Quarters','Interpreter','latex')
        ylim([-5 2])
        set(gca,'Ytick',-10:2:1)
    else
        ylim([-.2 2])
        set(gca,'Ytick',-10:1:10)
    end
    xlim([1 iH]);
    set(gca','Xticklabel',num2str((0:5:iH)'))
    set(gca,'Xtick',1:5:iH), xlim([1 iH]),
    
end





